@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # coding: utf-8
 
-# In[1]:
+# In[2]:
 
 
 try:
@@ -12,10 +12,10 @@ except:
         pass
 
 
-# In[100]:
+# In[3]:
 
 
-version = '0.59'
+version = '0.60'
 
 import re
 import tempfile
@@ -33,7 +33,7 @@ import pickle
 
 # # Helper Functions
 
-# In[101]:
+# In[4]:
 
 
 def is_testing():
@@ -43,7 +43,7 @@ def is_testing():
         return False
 
 
-# In[102]:
+# In[5]:
 
 
 def isnotebook():
@@ -54,7 +54,7 @@ def isnotebook():
     return False
 
 
-# In[103]:
+# In[6]:
 
 
 def testPrint(*args):
@@ -64,7 +64,7 @@ def testPrint(*args):
 
 # # Requiremtns
 
-# In[104]:
+# In[7]:
 
 
 requirements = []
@@ -93,7 +93,7 @@ from IPython.core.display import display, HTML
 
 # # Strings
 
-# In[105]:
+# In[8]:
 
 
 def sFloat0(f): return "{:0.0f}".format(f)
@@ -125,7 +125,7 @@ if is_testing():
 
 # # Process / Thread
 
-# In[359]:
+# In[9]:
 
 
 def mapThread(func, args, n=8):
@@ -148,14 +148,14 @@ if is_testing():
 
 # # Lists
 
-# In[314]:
+# In[10]:
 
 
 def liChunks(l, chunk_size):
     return [l[i:i + chunk_size] for i in range(0, len(l), chunk_size)]
 
 
-# In[315]:
+# In[11]:
 
 
 if is_testing():
@@ -164,7 +164,7 @@ if is_testing():
     print(liChunks(['a', 'b', 'c', 'd', 'e', 'f'], 3))
 
 
-# In[308]:
+# In[12]:
 
 
 def liSplit(a, number_of_splits):
@@ -172,7 +172,7 @@ def liSplit(a, number_of_splits):
     return [a[start:end] for start, end in zip(idxs[:-1], idxs[1:])]
 
 
-# In[309]:
+# In[13]:
 
 
 if is_testing():
@@ -184,7 +184,7 @@ if is_testing():
     print(liSplit(['a', 'b', 'c', 'd', 'e'], 6))
 
 
-# In[287]:
+# In[14]:
 
 
 def liJoin(l):
@@ -194,14 +194,14 @@ def liJoin(l):
     return out
 
 
-# In[289]:
+# In[15]:
 
 
 if is_testing():
     print(liJoin([['a', 'b', 'c'], ['d', 'e', 'f']]))
 
 
-# In[355]:
+# In[16]:
 
 
 def liFlatten(l):
@@ -218,7 +218,7 @@ def liFlatten(l):
         return l
 
 
-# In[358]:
+# In[17]:
 
 
 if is_testing():
@@ -228,7 +228,7 @@ if is_testing():
 
 # # List of Strings
 
-# In[110]:
+# In[18]:
 
 
 regex_cache = {}
@@ -238,35 +238,35 @@ def sListFilter(l, re_include=".*", re_exclude='^\b$'):
     return [e for e in l if regex_cache[re_include].search(e) and not regex_cache[re_exclude].search(e)]
 
 
-# In[111]:
+# In[19]:
 
 
 if is_testing():
     sListFilter(['good x', 'other good x', 'without the letter before y'])
 
 
-# In[112]:
+# In[20]:
 
 
 if is_testing():
     print(sListFilter(['good x', 'other good x', 'without the letter before y'], re_include='x'))
 
 
-# In[113]:
+# In[21]:
 
 
 if is_testing():
     print(sListFilter(['not enough x', 'enough xx', 'enough xx and also z'], re_include='xx+'))
 
 
-# In[114]:
+# In[22]:
 
 
 if is_testing():
     print(sListFilter(['not enough x', 'enough xx', 'enough xx and also z'], re_include='xx+', re_exclude='z'))
 
 
-# In[115]:
+# In[23]:
 
 
 def lCsv(l, force_oneline=False):
@@ -282,21 +282,21 @@ def lCsv(l, force_oneline=False):
     return ", ".join([str(e) for e in l])
 
 
-# In[116]:
+# In[24]:
 
 
 if is_testing():
     print(lCsv(['a', 'b', 1, 3.14]))
 
 
-# In[117]:
+# In[25]:
 
 
 if is_testing():
     print(lCsv([['a', 'b'], 1, 3.14]))
 
 
-# In[118]:
+# In[26]:
 
 
 if is_testing():
@@ -305,7 +305,7 @@ if is_testing():
 
 # # Text files
 
-# In[119]:
+# In[27]:
 
 
 def txtread(path):
@@ -326,7 +326,7 @@ def txtwrite(path, txt):
             return f.write(txt)
 
 
-# In[120]:
+# In[28]:
 
 
 if is_testing():
@@ -335,7 +335,7 @@ if is_testing():
 
 # # List of Strings
 
-# In[121]:
+# In[29]:
 
 
 regex_cache = {}
@@ -347,35 +347,35 @@ def sListFilter(listOfStrings, re_include=".*", re_exclude='^\b$'):
 
 # ### Examples
 
-# In[122]:
+# In[30]:
 
 
 if is_testing():
     print(sListFilter(['good x', 'other good x', 'without the letter before y']))
 
 
-# In[123]:
+# In[31]:
 
 
 if is_testing():
     print(sListFilter(['good x', 'other good x', 'without the letter before y'], re_include='x'))
 
 
-# In[124]:
+# In[32]:
 
 
 if is_testing():
     print(sListFilter(['not enough x', 'enough xx', 'enough xx and also z'], re_include='xx+'))
 
 
-# In[125]:
+# In[33]:
 
 
 if is_testing():
     print(sListFilter(['not enough x', 'enough xx', 'enough xx and also z'], re_include='xx+', re_exclude='z'))
 
 
-# In[126]:
+# In[34]:
 
 
 if is_testing():
@@ -384,7 +384,7 @@ if is_testing():
 
 # # GCP Google Cloud Platform
 
-# In[226]:
+# In[35]:
 
 
 import warnings
@@ -408,7 +408,7 @@ def _gsBlob(path):
     return blob
 
 
-# In[128]:
+# In[36]:
 
 
 def gsGsToHttp(path):
@@ -418,7 +418,7 @@ def gsHttpToGs(path):
     return path.replace("https://storage.cloud.google.com/", "gs://").split("?")[0] # "?" is for possible "?authuser=0"
 
 
-# In[375]:
+# In[37]:
 
 
 def gsLs(path):
@@ -428,7 +428,7 @@ def gsLs(path):
     return ['gs://' + bucket + '/' + blob.name for blob in client.list_blobs(prefix=prefix)]
 
 
-# In[377]:
+# In[38]:
 
 
 def gsExists(path):
@@ -438,7 +438,7 @@ def gsExists(path):
     return client.blob(prefix).exists()
 
 
-# In[130]:
+# In[39]:
 
 
 def gsUploadFile(gsPath, path):
@@ -446,7 +446,7 @@ def gsUploadFile(gsPath, path):
     blob.upload_from_filename(path)
 
 
-# In[131]:
+# In[40]:
 
 
 def gsTxtread(path):
@@ -458,7 +458,7 @@ def gsTxtread(path):
     return blob.download_as_string().decode()
 
 
-# In[132]:
+# In[41]:
 
 
 def gsTxtwrite(path, s):
@@ -466,7 +466,7 @@ def gsTxtwrite(path, s):
     return blob.upload_from_string(s)
 
 
-# In[133]:
+# In[42]:
 
 
 def gsImwrite(path, img):
@@ -475,7 +475,7 @@ def gsImwrite(path, img):
     blob.upload_from_string(encimg.tobytes())
 
 
-# In[134]:
+# In[43]:
 
 
 def gsImread(path):
@@ -484,7 +484,7 @@ def gsImread(path):
     return cv2.imdecode(raw, 1)[:, :, [2, 1, 0]]
 
 
-# In[135]:
+# In[44]:
 
 
 def gsPklWrite(path, obj):
@@ -492,7 +492,7 @@ def gsPklWrite(path, obj):
     return blob.upload_from_string(pickle.dumps(obj, protocol=4))
 
 
-# In[136]:
+# In[45]:
 
 
 def gsPklRead(path):
@@ -502,7 +502,7 @@ def gsPklRead(path):
 
 # # Pickle
 
-# In[137]:
+# In[46]:
 
 
 def pklwrite(path, obj):
@@ -513,7 +513,7 @@ def pklwrite(path, obj):
             return pickle.dump(obj, f, protocol=4)
 
 
-# In[138]:
+# In[47]:
 
 
 def pklread(path):
@@ -524,7 +524,7 @@ def pklread(path):
             return pickle.load(f)
 
 
-# In[139]:
+# In[48]:
 
 
 if is_testing():
@@ -537,7 +537,7 @@ if is_testing():
 
 # # Images
 
-# In[140]:
+# In[49]:
 
 
 def imread(path):
@@ -571,7 +571,7 @@ def impad(img, top=0, bottom=0, left=0, right=0, color=255):
     return np.pad(img, [(top, bottom), (left, right), (0, 0)], 'constant', constant_values=color)
 
 
-# In[141]:
+# In[50]:
 
 
 def imshow(array, scale=1):
@@ -586,7 +586,7 @@ def imshow(array, scale=1):
         cv2.waitKey()
 
 
-# In[142]:
+# In[51]:
 
 
 if is_testing():
@@ -594,14 +594,14 @@ if is_testing():
     imshow(img)
 
 
-# In[143]:
+# In[52]:
 
 
 if is_testing():
     imshow('http://via.placeholder.com/70.png')
 
 
-# In[144]:
+# In[53]:
 
 
 def imscaleNN(img, s):
@@ -611,7 +611,7 @@ def imscaleBic(img, s):
     return cv2.resize(img, None, fx=s, fy=s, interpolation=cv2.INTER_CUBIC)
 
 
-# In[145]:
+# In[54]:
 
 
 if is_testing():
@@ -630,7 +630,7 @@ if is_testing():
         imshow(out_large_interbic)
 
 
-# In[146]:
+# In[55]:
 
 
 def imRepeat(img, height, width):
@@ -644,7 +644,7 @@ def imRepeat(img, height, width):
     return img
 
 
-# In[147]:
+# In[56]:
 
 
 if is_testing():
@@ -652,33 +652,33 @@ if is_testing():
     imshow(imRepeat(img_random_color_3x5, 7, 11), scale=10)
 
 
-# In[148]:
+# In[57]:
 
 
 def imNewWhite(height, width):
     return np.ones((height, width, 3)).astype(np.uint8) * 255
 
 
-# In[149]:
+# In[58]:
 
 
 if is_testing(): imshow(imNewWhite(10, 500) - 30)
 
 
-# In[150]:
+# In[59]:
 
 
 def imNewBlack(height, width):
     return np.zeros((height, width, 3)).astype(np.uint8)
 
 
-# In[151]:
+# In[60]:
 
 
 if is_testing(): imshow(imNewBlack(10, 500))
 
 
-# In[152]:
+# In[61]:
 
 
 def imAddNoiseGauss(img, std):
@@ -690,7 +690,7 @@ def imAddNoiseGauss(img, std):
 if is_testing(): imshow(imAddNoiseGauss(imread('http://via.placeholder.com/150.png'), 10))
 
 
-# In[153]:
+# In[62]:
 
 
 def imJpgDegradation(img, quality):
@@ -703,7 +703,7 @@ def imJpgDegradation(img, quality):
 if is_testing(): imshow(imJpgDegradation(imread('http://via.placeholder.com/150.png'), 15))
 
 
-# In[154]:
+# In[63]:
 
 
 def imCropCenter(img, size):
@@ -718,35 +718,35 @@ def imCropCenter(img, size):
     return img[h_start:h_end, w_start:w_end]
 
 
-# In[155]:
+# In[64]:
 
 
 if is_testing(): 
     imshow(imCropCenter(imread('http://via.placeholder.com/40.png'), 10))
 
 
-# In[156]:
+# In[65]:
 
 
 if is_testing(): 
     imshow(imCropCenter(imread('http://via.placeholder.com/40.png'), 49))
 
 
-# In[157]:
+# In[66]:
 
 
 if is_testing(): 
     imshow(imCropCenter(imread('http://via.placeholder.com/40.png'), 50))
 
 
-# In[158]:
+# In[67]:
 
 
 if is_testing(): 
     imshow(imCropCenter(imread('http://via.placeholder.com/40.png'), 100))
 
 
-# In[159]:
+# In[68]:
 
 
 def imGallery(imgs, pad=0):
@@ -777,7 +777,7 @@ def imGallery(imgs, pad=0):
     return out
 
 
-# In[160]:
+# In[69]:
 
 
 if is_testing():
@@ -785,49 +785,49 @@ if is_testing():
     listOfDummyImagesBlackToWhite = [imNewBlack(10, 10) + 25 * i for i in range(10)]
 
 
-# In[161]:
+# In[70]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:1]))
 
 
-# In[162]:
+# In[71]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:2]))
 
 
-# In[163]:
+# In[72]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:2], pad=1))
 
 
-# In[164]:
+# In[73]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:5], pad=1))
 
 
-# In[165]:
+# In[74]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:9], pad=1))
 
 
-# In[166]:
+# In[75]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:10], pad=1))
 
 
-# In[167]:
+# In[ ]:
 
 
 if is_testing():
@@ -837,14 +837,14 @@ if is_testing():
 
 # # Files
 
-# In[168]:
+# In[ ]:
 
 
 def fiFindByWildcard(wildcard):
     return natsort.natsorted(glob.glob(wildcard, recursive=True))
 
 
-# In[169]:
+# In[ ]:
 
 
 if is_testing():
@@ -873,7 +873,7 @@ if is_testing():
         imshow(imGallery(imgs))
 
 
-# In[170]:
+# In[ ]:
 
 
 def fiRemove(path):
@@ -901,7 +901,7 @@ def fiAgeHours(path):
     return (time.time() - os.stat(path)[stat.ST_MTIME]) / 3600
 
 
-# In[171]:
+# In[ ]:
 
 
 if is_testing():
@@ -933,9 +933,34 @@ if is_testing():
         print("files: " + "\n".join(fiFindByWildcard(os.path.join(d, "**", "*"))))
 
 
+# In[ ]:
+
+
+def fiUnzip(zip_path, out_dir=None, remove=False):
+    import zipfile, shutil
+    out_dir = zip_path.replace(".zip", "") if out_dir is None else out_dir
+    assert out_dir != zip_path, zip_path
+    
+    if remove:
+        shutil.rmtree(out_dir, ignore_errors=True)
+        
+    with zipfile.ZipFile(zip_path, 'r') as zip:
+        zip.extractall(out_dir)
+
+
+# In[ ]:
+
+
+def fiJoin(*args):
+    return os.path.join(*args)
+
+if is_testing():
+    print(fiJoin('path', 'to'))
+
+
 # # Jupyter
 
-# In[172]:
+# In[ ]:
 
 
 def jnNbFullWidth():
@@ -949,7 +974,7 @@ def jnPandasSettings():
     pd.set_option('display.max_colwidth', None)
 
 
-# In[173]:
+# In[ ]:
 
 
 def jnImageRenderingPixelated():
@@ -961,7 +986,7 @@ def jnImageRenderingAuto():
     display(HTML('<style type = text/css> img { image-rendering: auto; } </style>'))
 
 
-# In[174]:
+# In[ ]:
 
 
 def jnDefault():
@@ -973,7 +998,13 @@ if is_testing():
     jnDefault()
 
 
-# In[175]:
+# In[ ]:
+
+
+
+
+
+# In[ ]:
 
 
 def jnHtmlRow(strList, width=100):
@@ -986,7 +1017,7 @@ if is_testing():
     
 
 
-# In[176]:
+# In[ ]:
 
 
 def jnImgToHtmlB64(img):
@@ -1021,7 +1052,7 @@ def jnImageRow(imgs, names=None, width=100, external=False, title_img=None, titl
     display(HTML(html))
 
 
-# In[177]:
+# In[ ]:
 
 
 if is_testing():
@@ -1029,7 +1060,7 @@ if is_testing():
    
 
 
-# In[178]:
+# In[ ]:
 
 
 if is_testing():
@@ -1039,7 +1070,7 @@ if is_testing():
     jnImageRow(imgs, external=True, title='test')
 
 
-# In[179]:
+# In[ ]:
 
 
 
@@ -1049,7 +1080,7 @@ if is_testing():
 
 # # Shell
 
-# In[180]:
+# In[ ]:
 
 
 def shell(cmd):
@@ -1064,7 +1095,7 @@ def shell(cmd):
 
 # # GIT
 
-# In[181]:
+# In[ ]:
 
 
 def gitGetHash():
@@ -1074,7 +1105,7 @@ if is_testing():
     print(gitGetHash())
 
 
-# In[182]:
+# In[ ]:
 
 
 def gitGetBranch():
@@ -1084,7 +1115,7 @@ if is_testing():
     print(gitGetBranch())
 
 
-# In[183]:
+# In[ ]:
 
 
 def gitGetMessage():
@@ -1094,7 +1125,7 @@ if is_testing():
     print(gitGetMessage())
 
 
-# In[184]:
+# In[ ]:
 
 
 def gitGetAllFiles():
@@ -1104,7 +1135,7 @@ if is_testing():
     print(gitGetAllFiles())
 
 
-# In[185]:
+# In[ ]:
 
 
 def gitListByTime(path=None):
@@ -1120,7 +1151,7 @@ if is_testing():
     print(gitListByTime()[-3:])
 
 
-# In[186]:
+# In[ ]:
 
 
 def shellSortDirsBySize():
@@ -1132,14 +1163,14 @@ if is_testing():
 
 # # Package Files
 
-# In[187]:
+# In[ ]:
 
 
 if is_testing():
     package_files = {}
 
 
-# In[188]:
+# In[ ]:
 
 
 def shell(cmd):
@@ -1147,7 +1178,7 @@ def shell(cmd):
     return result.stdout.decode()
 
 
-# In[189]:
+# In[ ]:
 
 
 if is_testing():
@@ -1179,7 +1210,7 @@ setup(
 )""".format(requirements=", ".join(["'" + r + "'" for r in requirements]), version=version)
 
 
-# In[190]:
+# In[ ]:
 
 
 if is_testing():
@@ -1189,7 +1220,7 @@ description-file = README.md
 """
 
 
-# In[191]:
+# In[ ]:
 
 
 if is_testing():
@@ -1212,14 +1243,14 @@ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 SOFTWARE."""
 
 
-# In[192]:
+# In[ ]:
 
 
 if is_testing():
     package_files['README.md'] = ""
 
 
-# In[193]:
+# In[ ]:
 
 
 if is_testing():
@@ -1228,7 +1259,7 @@ if is_testing():
     
 
 
-# In[194]:
+# In[ ]:
 
 
 if is_testing():
@@ -1239,7 +1270,7 @@ lx/__init__.py
 """
 
 
-# In[195]:
+# In[ ]:
 
 
 if is_testing():
